/* Copyright (c) Colorado School of Mines, 2008.*/
/* All rights reserved.                       */

/* SUTTOZ: $Revision: 1.17 $ ; $Date: 2008/01/21 16:44:13 $		*/
#include <cstddef>

//void ints8r (int nxin, float dxin, float fxin, float yin[], float yinl, float yinr, int nxout, float xout[], float yout[]);
//void yxtoxy (int nx, float dx, float fx, float y[], int ny, float dy, float fy, float xylo, float xyhi, float x[]);

#include "cwp.h"

/*********************** self documentation ******************************/
char *sdoc[] = {
"									",
" SUTTOZ - resample from time to depth					",
"									",
" suttoz <stdin >stdout [optional parms]				",
"									",
" Optional Parameters:							",
" nz=1+(nt-1)*dt*vmax/(2.0*dz)   number of depth samples in output	",
" dz=vmin*dt/2		depth sampling interval (defaults avoids aliasing)",
" fz=v(ft)*ft/2		first depth sample				",
" t=0.0,...		times corresponding to interval velocities in v",
" v=1500.0,...		interval velocities corresponding to times in v",
" vfile=		  binary (non-ascii) file containing velocities v(t)",
" verbose=0		>0 to print depth sampling information		",
"									",
" Notes:								",
" Default value of nz set to avoid aliasing				",
" The t and v arrays specify an interval velocity function of time.	",
"									",
" Note that t and v are given  as arrays of floats separated by commas,  ",
" for example:								",
" t=0.0,0.01,.2,... v=1500.0,1720.0,1833.5,... with the number of t values",
" equaling the number of v values. The velocities are linearly interpolated",
" to make a continuous, piecewise linear v(t) profile.			",
"									",
" Linear interpolation and constant extrapolation is used to determine	",
" interval velocities at times not specified.  Values specified in t	",
" must increase monotonically.						",
"									",
" Alternatively, interval velocities may be stored in a binary file	",
" containing one velocity for every time sample.  If vfile is specified,",
" then the t and v arrays are ignored.					",
"									",
" see selfdoc of suztot  for depth to time conversion			",
"									",
" Trace header fields accessed:  ns, dt, and delrt			",
" Trace header fields modified:  trid, ns, d1, and f1			",
"									",
NULL};

/* Credits:
 *	CWP: Dave Hale c. 1992
 *
 */
/**************** end self doc *******************************************/

/* functions defined and used internally */
static void maketz (int nt, float dt, float ft, float v[], 
	int nz, float dz, float fz, float t[]);
static void zttz(int nt, float dt, float ft, float zt[], float vft, float vlt, 
	int nz, float dz, float fz, float tz[]);

int
main(int argc, char **argv)
{
    int nt,nz;
    float dt,ft,dz,fz;
    float *vt=new float[nt];
    float *datain=new float[nt];
    float *tz=new float[nz];
    float *dataout=new float[nz];

	fprintf(stderr,"haha\n");
	maketz(nt,dt,ft,vt,nz,dz,fz,tz);

    int n=10;
	/* loop over traces */
	for(int i=0;i<n;i++) {
		ints8r(nt,dt,ft,datain,0.0,0.0,nz,tz,dataout);
	}
    
    delete []vt;delete []datain;delete []tz;delete []dataout;
	return 0;
}

static void maketz (int nt, float dt, float ft, float v[], 
	int nz, float dz, float fz, float t[])
/************************************************************************
maketz - compute t(z) from v(t)
*************************************************************************
Input:
nt	number of time samples
dt	time sampling interval
ft	first time sample
v[]	array of velocities as a function of time t
nz	number of z values (output)
dz	depth sampling interval (output)
fz	first depth value (output)
Output:
t[]	array of t values as a function of z
*************************************************************************
Author: CWP: Dave Hale (c. 1992)
*************************************************************************/
{
	int it;			/* counter */
	float vft;		/* velocity at the first time sample */
	float vlt;		/* velocity at the last time sample */
	float *z=NULL;		/* array of depth values as a function of t */

	/* allocate space */
	z = new float[nt];

	/* calculate z(t) from v(t) */
	z[0] = 0.5*ft*v[0];
	for (it=1; it<nt; it++)
		z[it] = z[it-1]+0.5*dt*v[it-1];
	vft = v[0];
	vlt = v[nt-1];

	/* compute t(z) from z(t) */
	zttz(nt,dt,ft,z,vft,vlt,nz,dz,fz,t);
    delete []z;
}

static void zttz(int nt, float dt, float ft, float zt[], float vft, float vlt, 
	int nz, float dz, float fz, float tz[])
/************************************************************************
zttz - compute t(z) from z(t)
*************************************************************************
Input:
nt	number of time samples
dt	time sampling interval
ft	first time value
zt[]	array of z values as a function of time
vft	velocity at first time sample
vlt	velocity at last time sample
nz	number of z values  (output)
dz	depth sampling interval (output)
fz	first z value
Output:
tz[] 	array of time values as a function of depth
*************************************************************************
Author: CWP: Dave Hale (c. 1992)
************************************************************************/
{
	int iz;				/* depth counter */
	float z;			/* depth */
	float lt=ft+(nt-1)*dt;		/* last time */
	float lz=fz+(nz-1)*dz;		/* last depth */

	/* switch from z(t) to t(z) */
	yxtoxy(nt,dt,ft,zt,nz,dz,fz,0.0,0.0,tz);

	/* for z values before fz, use first velocity to calculate t(z) */
	for (iz=0,z=fz; z<=zt[0]; iz++,z+=dz)
		tz[iz] = 2.0*z/vft;

	/* for z values from lz down to fz, calculate t(z) */
	for (iz=nz-1,z=lz; z>=zt[nt-1]; iz--,z-=dz)
		tz[iz] = lt+2.0*(z-zt[nt-1])/vlt;
}
